% Script for the hybrid wind instrument
% version 1.3

clc
clearvars
close all

%% %!set the instrument mode and the desired excitation model:

instrument_mode = 'simulation';
%instrument_mode = 'hybrid';

%Main excitation models %!select
%excitation_model = 'single_reed';
excitation_model = 'lip_reed';
%excitation_model = 'bowed_string_hyperbolic';
%excitation_model = 'polynomial';

%Other models, to test, etc.
%excitation_model = 'measurements'; %to study the open loop
%excitation_model = 'single_reed_wg_p_test'; %test with waveguide
%excitation_model = 'free_reed';
%excitation_model = 'single_reed_decayed';
%excitation_model = 'lip_reed_auto_tuned';
%excitation_model = 'fletcher_reeds';
%excitation_model = 'double_reed';
%excitation_model = 'bowed_string_weinreich';
%excitation_model = 'bowed_string_2pole2zero';
%excitation_model = 'linfbtest';

%% Data parameters

%%%% Global parameters %%%%

% Solver
Ts = 25e-6; %(s) solver period
fs = 1/Ts;%(Hz) sampling frequency for 
runtime = 5 %(s) 

% [sample_orig,fs_sample,NBITS]=wavread('RoosterSample.wav');
% sample.signals.values=resample(sample_orig,fs,fs_sample);
% sample.signals.values=sample.signals.values(:,1);
% sample.time=(0:length(sample.signals.values)-1)'*Ts;

% Low-pass filter to prevent erroneous feedback:
fLPcut =10000;%[Hz]
hLP = fdesign.lowpass('N,F3dB',2,fLPcut,fs); %! I might try other LP filters with less phase lag... but they're less steep...
HdLP = design(hLP, 'butter');
%! may have an influence on the normal functioning as well, especially
%because of the phase...
%-> -90deg @ cut-off freq, �linear progression.

%Resonator Parameters:
rho = 1.204; %(kg/m^3) air volumetric mass
c = 344; %(m/s) the speed of sound (21.5 C)
% Tube characteristic impedance :
a = 14.2e-3/2  % (m) tube section radius: 14.2e-3/2; %real clarinet: 15.0e-3/2 %! set!
St = a^2 *pi; % (m^2) tube cross sectional area
Zc = rho*c/St;

%Vibrato simulation with variable virtual parallel compliance at the
%instrument input
% Low-pass filter to prevent erroneous feedback:
vibrato.amp = 0.02e-3; %seconds added to the cycle
vibrato.freq = 50; %Hz
fLPcut2 =1000;%[Hz]
hLP2 = fdesign.lowpass('N,F3dB',2,fLPcut2,fs); %! I might try other LP filters with less phase lag... but they're less steep...
HdLP2 = design(hLP2, 'butter');
fLPcut3 =2;%[Hz]
hLP3 = fdesign.lowpass('N,F3dB',2,fLPcut3,fs); %! I might try other LP filters with less phase lag... but they're less steep...
HdLP3 = design(hLP3, 'butter');

%Excitation model:
% Execute script with the excitation model:
run(excitation_model)
    
if strcmp(instrument_mode,'simulation')
    % Modal tube approach -> impedance measurement approximated by iterative
    %modal script... -> load the result:
    % for coupled LS-tube, use: modal_coeff_Zvirt_LS_hybridtube_OL (+see swap
    % simulink file for the right blocks...)
    load('../../../Data/Characterising/Calculated/Impedances/Modal coefficients/modal_coeff_vibro_pt1-2.mat','modal_coeff');
    an = modal_coeff(:,1);
    wn = modal_coeff(:,2);
    Qn = modal_coeff(:,3);
%     %!just for test: using delay line approach:
%     Lt=58e-2;
%     % pour le rayonnement (tube sans ecran) (AIM p.590):
%     delta_ray = 0.6133;
%     beta_loss = 1/4;
%     n1 = 0.167;
%     d1 = 2*delta_ray + n1;
%     d2 = (d1^2 - n1^2)/2 - 2*beta_loss;
%     %losses
%     dL = 0.896;
%     TUBE_REF_c = tf([-n1*a/c -1],[d2*(a/c)^2 d1*a/c 1]);
%     TUBE_REF_d = c2d(TUBE_REF_c,Ts);%(! this uses the "zoh" method.

elseif strcmp(instrument_mode,'hybrid')
    temp = 22.6; %!set
    % Saving data parameters
    lastbuffer_time = runtime;%[s] saving buffer time (last buffer recorded) %!set
    decim = 1;% decimation value for saving data
    Te = decim*Ts; %[s] sampling period for saved data
    fe = 1/Te; % saved data sampling frequency
    lastbuffer_samps = lastbuffer_time*fe; % number of samples in saved buffer
    lastbuffer_samps_rnd = ceil(lastbuffer_samps/1024)*1024; % up-rounded kb multiple number of samples for saving data
    %Amplifier parameters:
    maxlinevoltage=0.447;
    maxoutputvoltage = 5;
    % Microphone, loudspeaker and amplifier parameters
    micpreampsensitivity =  -0.0025726 %!! For Knowles mic (in the centre): 0.003272 %for B&K mic use: -0.0025726 %last calibration:23/9/2015 % V/Pa % the minus sign is to correct for the inverted output... % for the first mic, we used: -2.6393e-3; % for the measurements of ISMA2014: -0.0025478;
    %! Perform an amplifier gain calibration first! put the found factor here:
    amp_gain_factor =   0.73316;%last calibration:%23/9/2016; 21/5/2016 %23/9/2015  0.56..=prototype 1.1.1 & 1.2 %prototype 1.1: 6.5951 %amp_gain_factor =  1.9733 % V/V % for the measurements with the first mic, we used 2.0776 % for the measurements of ISMA2014: 2.0264
    %if laser vibrometer used and connected to xenomai computer:
    %vibro_sens = 1/10e-3;%V/(m/s)
    %Loudspeaker Parameters:
    % load LS estimated parameters for the loudspeaker compensating filters
    load('../../../Data/Characterising/Calculated/LS Parameter estimation/Estimated_LS_parameters_vibro_pt1-2.mat')
    %load('../../../Data/Characterising/Calculated/LS Parameter estimation/Estimated_LS_parameters_vibro_pt2-0.mat')

%     %! Trying to include the coil induction!:
%     Le = 2.2649e-04;
%     Re = 6.0501;
%     Bl = 3.1252;
%     %using manual "euler backward" discretisation;
%     He_d = tf([Le+Ts*Re, -Le],[Ts*Bl 0],Ts); %after 2kHz (with fs=40kHz),
%     %the phase already drops again...
%     %Consider bilinear approach:
%     He_c = tf([Le Re],Bl);
%     He_d_bilin = c2d(He_c,Ts,'tustin'); %! but that seems to increase the gain too much towards the Nyquist frequency, probably going to cause instability)
%     %consider to use a current amplifier which will automatically
%     %compensate for the coil inductance! For additional increasing phase
%     %offset, an analogue differentiator can be better considered.
	
    %Calculate lead-lag discrete filter variables:
    HLL_c = tf([1 LLwresn/LLQnd LLwresn^2],[1 LLwresd/LLQnd LLwresd^2]);
    HLL_d = c2d(HLL_c,Ts);%(! this uses the "zoh" method. however I wrote I use the Euler backward method (Buys2015Acustica)... but matlab doesn't seem to provide that! ("match" is Euler forward I guess). But those bode plots look all the same, so no worries!
    %Controller settings:
    %linear scaling:
    % Balance
    balance1 = 2.41;balance2 = 0.64;
    bal2gamm.slope=(par1controlmax-par1controlmin)/(balance2-balance1);
    bal2gamm.offset=bal2gamm.slope*(-balance1)+par1controlmin;
    % LSR sensor
    lsr1 = 0;lsr2 = 7;%hmight be different for more light...
    lsr2zeta.slope=(par2controlmax-par2controlmin)/(lsr2-lsr1);
    lsr2zeta.offset=lsr2zeta.slope*(-lsr1)+par2controlmin;
    % Mouth air pressure
    mouth1 = 1.5;mouth2 = 10; %!to calibrate
    mouth2gamm.slope=(par1controlmax-par1controlmin)/(mouth2-mouth1);
    mouth2gamm.offset=mouth2gamm.slope*(-mouth1)+par1controlmin;
    % Lip load
    lip1 = 0;lip2 = 10;%!to calibrate
    lip2zeta.slope=(par2controlmax-par2controlmin)/(lip2-lip1);
    lip2zeta.offset=lip2zeta.slope*(-lip1)+par2controlmin;
    % Lip load to reed resonance
    lip1 = 0;lip2 = 10;%!to calibrate
    lip2freed.slope=(par2controlmax-par2controlmin)/(lip2-lip1);
    lip2freed.offset=lip2freed.slope*(-lip1)+par2controlmin;
    % Lip load to reed resonance
    lip1 = 0;lip2 = 10;%!to calibrate
    lip2vibratofreq.slope=(150-0)/(lip2-lip1);
    lip2vibratofreq.offset=lip2vibratofreq.slope*(-lip1)+0;
end

%% Runtime & post procedures:

if strcmp(instrument_mode,'simulation')
    % Launch the simulation with these parameters :
    sim(['simu_wind_' excitation_model '.mdl']);
    p_dl = p_dl.signals.values;
    q_dl = q_dl.signals.values;
    %zeta_vect_simu = zeta_vect_simu.signals.values;
    %gamma_vect_simu = gamma_vect_simu.signals.values;
    % Save all acquisition parameters & results:
    timevect = (0:Ts:runtime)';
    plotlinespecs = '--r';
    save(['Acquisition_parameters_and_results_simulation_' excitation_model]);
elseif strcmp(instrument_mode,'hybrid')
    %activate driver (actually needed just once, but does no harm executing each time...)
    unix('echo xenomai | sudo -S /usr/xenomai/sbin/analogy_config analogy0 analogy_ni_pcimio 0x4,0');
    %run the calibration program:
    unix('./calib_micoffset -time 3');
    load('calib_micoffset.mat');
    voltsin_calib = rt_yout(:,2); ampin_volts_calib = rt_yout(:,3); 
    micoffset_V=mean(voltsin_calib);
    %build the c code:
    rtwbuild(['hybrid_wind_' excitation_model])
    %run the xenomai patch:
    unix(['echo xenomai | sudo -S ./moulinette.sh hybrid_wind_' excitation_model]);
    %execute the program:
    unix(['echo xenomai | sudo -S ./hybrid_wind_' excitation_model ' -time ' num2str(runtime)]);
    %load the results and put it into appropriate vectors:
    load(['hybrid_wind_' excitation_model '.mat'])
    timevect = rt_tout(:,1);
    p_dl = rt_yout(:,1);
    q_dl = rt_yout(:,2);
    ampin_volts = rt_yout(:,3);
    nclipped=sum(abs(ampin_volts)-maxoutputvoltage>=0);
    if nclipped>0
        warning('AMPLIFIER SIGNAL HAS CLIPPED %d TIMES!',nclipped)
        figure,
        plot(timevect,ampin_volts)
    end
    clear rt_yout rt_tout;
    plotlinespecs = '-b';
    save(['Acquisition_parameters_and_results_hybrid_' excitation_model]);
end

%% The sound of the mouthpiece pressure signal
% At the resonator entrance:
fs_sound = fs;%44100;
p_sound=resample(p_dl,fs_sound,round(fs));% only exactly the right timing when round(fs)=fs ...
p_sound = p_sound/max(abs(p_sound));

% Play the sound, note: we here some aliasing -> due to the Matlab sound
% there is no aliasing in the p_sound vector !
%sound(p_sound,fs_sound)
%wavwrite(p_sound,fs_sound,'p')
%!for newer matlab:
audiowrite([excitation_model '_p_' instrument_mode '.wav'],p_sound,fs_sound)

% Calculate approximate radiated sound: pext=d(p_dl+q_dl)/dt
pq=p_dl+q_dl;
% low-pass filter: Construct an FDESIGN object and call its BUTTER method.
hLP4k  = fdesign.lowpass(3752, 6000, 1, 120, fs);%Fpass, Fstop, Apass, Astop, Fs
HdLP4k = design(hLP4k, 'butter', 'MatchExactly', 'passband');
pqLP = filter(HdLP4k, pq); %filter the data vector
%derive:
dpq = diff(pqLP); dt=diff(timevect);
pext = dpq./dt;% note that only relative pext amplitudes are relevant... -> but its only important to have dimensionless ENVrms output... which uses p and not pext
pext = [pext; 0];%.*PMout.signals.values;%! take into account PM variations now
pext= pext/max(abs(pext));
%wavwrite(pext,fs_sound,'p_ext')
%for newer matlab:
audiowrite([excitation_model '_p_ext_' instrument_mode '.wav'],pext,fs_sound)

sound(pext,fs_sound)


%% Figures, complete signals

% Mouthpiece pressure signal over time
figuren('p'),
plot(timevect,p_dl,plotlinespecs)%gamma.signals.values-p_dl %(0:1:(length(p_dl)-1))/fs,
grid on,
xlabel('Time (s)')
ylabel('Dimensionless pressure p (Pa/P_M)')
hold on,

figuren('q'),
plot(timevect,q_dl,plotlinespecs)%-mean(q_dl(end/2:end))
hold on,

if strcmp(instrument_mode,'hybrid')
    figuren('p_hyb'),
    plot(timevect,ampin_volts,plotlinespecs)
    hold on,
    plot(timevect,U_LS.signals.values*Zc/PM,'--r') %(!
    plot(timevect,p_dl,':k')
end


%% Quick preview of a pressure and flow rate signal + pressure spectrum
run('../../Describing_sounds/audio_descriptors/pressure_snapshots.m')%(!adapt script

%% Timbre analysis with the MATIMBRE toolbox
%-> go to these scripts:
%audiodescriptors_matimbre_sustained_sounds.m;
%audiodescriptors_matimbre_attack_sounds.m;
