clc
clear all
close all

%%

%%%% Global parameters %%%%

% Solver
Ts = 25e-6; %[s] solver period
fs = 1/Ts;%[s] sampling period for solver
runtime = 20;

% Low-pass filter to prevent erroneous feedback:
fLPcut =10000;%[Hz]
hLP = fdesign.lowpass('N,F3dB',2,fLPcut,fs); %! I might try other LP filters with less phase lag... but they're less steep...
HdLP = design(hLP, 'butter');

temp = 24.5; %!set
% Saving data parameters
lastbuffer_time = runtime;%[s] saving buffer time (last buffer recorded) %!set
decim = 1;% decimation value for saving data
Te = decim*Ts; %[s] sampling period for saved data
fe = 1/Te; % saved data sampling frequency
lastbuffer_samps = lastbuffer_time*fe; % number of samples in saved buffer
lastbuffer_samps_rnd = ceil(lastbuffer_samps/1024)*1024; % up-rounded kb multiple number of samples for saving data
%Amplifier parameters:
maxlinevoltage=0.447;
maxoutputvoltage = 5;
% Microphone, loudspeaker and amplifier parameters
micpreampsensitivity = -0.0025726 %!for Knowles mic (in the centre)  0.003272 %for B&K mic use: -0.0025726%! %last calibration:23/9/2015 %for +20dB preamp, do: /10  % V/Pa % the minus sign is to correct for the inverted output... % for the first mic, we used: -2.6393e-3; % for the measurements of ISMA2014: -0.0025478;
%! Perform an amplifier gain calibration first! put the found factor here:
amp_gain_factor = 0.73312; %last calibration:22/3/'16 %last calibration:23/9/2015 0.657  0.56..=prototype 1.1.1 & 1.2 %prototype 1.1: 6.5951 %amp_gain_factor =  1.9733 % V/V % for the measurements with the first mic, we used 2.0776 % for the measurements of ISMA2014: 2.0264
%if laser vibrometer used and connected to xenomai computer:
%vibro_sens = 1/10e-3;%V/(m/s)
%Loudspeaker Parameters:
% load LS estimated parameters for the loudspeaker simulation
LSpars = load('../../../Data/Characterising/Calculated/LS Parameter estimation/Estimated_LS_parameters_vibro_pt1-2.mat')
%!!LSpars = load('../../../Data/Characterising/Calculated/LS Parameter estimation/Estimated_LS_parameters_vibro_pt2-0.mat')
%for simplicity also load this way for the moment:
load('../../../Data/Characterising/Calculated/LS Parameter estimation/Estimated_LS_parameters_vibro_pt1-2.mat')
%!!load('../../../Data/Characterising/Calculated/LS Parameter estimation/Estimated_LS_parameters_vibro_pt2-0.mat')


HLL_c = tf([1 LLwresn/LLQnd LLwresn^2],[1 LLwresd/LLQnd LLwresd^2]);
HLL_d = c2d(HLL_c,Ts);%(! this uses the "zoh" method. however I wrote I use the Euler backward method (Buys2015Acustica)... but matlab doesn't seem to provide that! ("match" is Euler forward I guess). But those bode plots look all the same, so no worries!

%Resonator Parameters:
rho = 1.204; %(kg/m^3) air volumetric mass
c = 344; %(m/s) the speed of sound (21.5 C)
% Tube characteristic impedance :
a = 14.2e-3/2  % (m) tube section radius: 14.2e-3/2; %real clarinet: 15.0e-3/2 %! set!
St = a^2 *pi; % (m^2) tube section surface (cas du tube blanc au labo...)
Zc = rho*c/St;

%%
% chirp for measurements
fstart = 20; % Frequence basse du chirp
fstop = 10000; % Frequence haute du chirp
sweep_time = runtime; % Duree (en s)
Ampl_chirp = maxoutputvoltage/4;

Ampl_chirp_U = maxoutputvoltage *amp_gain_factor*Bl/Re/(2*pi*fstop)*Sd/Mms; % for cte flow rate sweep
maxLSvolts = 0.3;%!!0.3 %roughly corresponds to a hybrid operation with PM=100Pa for the f0 component (for umax=2/(3*sqrt(3)) and VLSmax = umax*Mms*2*pi*f0/Sd*Re/Bl = 0.27)
Ampl_chirp_F = maxLSvolts*Bl/Re; %! but: feedback not taken into account! % for cte force(/voltage) sweep

%% prepare xenomai & code
%activate driver (actually needed just once, but does no harm executing each time...)
unix('echo xenomai | sudo -S /usr/xenomai/sbin/analogy_config analogy0 analogy_ni_pcimio 0x4,0');%!!? correct? - sometimes it doesn't work... maybe problem with password technique or so...
    %run the calibration program:
    unix('./calib_micoffset -time 3'); %! note: this uses a fixed micSensitivity! => better remove offset as voltage not in Pa!!
    load('calib_micoffset.mat');
    voltsin_calib = rt_yout(:,2); ampin_volts_calib = rt_yout(:,3); 
    micoffset_V=mean(voltsin_calib);
%build the c code:
rtwbuild('hybrid_wind_measurements')
%run the xenomai patch:
unix(['echo xenomai | sudo -S ./moulinette.sh hybrid_wind_measurements']);
%% execute the program
unix(['echo xenomai | sudo -S ./hybrid_wind_measurements -time ' num2str(runtime)]);
%load the results and put it into appropriate vectors:
load(['hybrid_wind_measurements.mat'])
timevect = rt_tout(:,1);
chirpsignal = rt_yout(:,1);
pressure_Pa = rt_yout(:,2);
ampin_volts = rt_yout(:,3);

nclipped=sum(abs(ampin_volts)-maxoutputvoltage>=0);
if nclipped>0
    warning('AMPLIFIER SIGNAL HAS CLIPPED %d TIMES!',nclipped)
    figure,
    plot(timevect,ampin_volts)
end
clear rt_yout rt_tout;
save('Measurement_parameters_and_results_hybrid_measurements');

%% Time signals

% Mouthpiece pressure signal over time
figure,
plot(timevect,ampin_volts)%gamma.signals.values-p_dl %!?(0:1:(length(p_dl)-1))/fs,
grid on,
xlabel('Time (s)')
ylabel('Volts to amplifier (V)')
hold on,

figure,
plot(timevect,pressure_Pa)
xlabel('Time (s)')
ylabel('Pressure at microphone (Pa)')
hold on,

%% Spectra

% Prepare spectra
% The pressure spectrum
[P,freq_v] = td2fd(pressure_Pa,fs);
% The amp input voltage spectrum
UAO = td2fd(ampin_volts,fs)*amp_gain_factor;
% Backwards calculate Qtilde from UAO and P
Q_tilde = HLSinv(UAO,P,LSpars,Ts,freq_v);%! Hderiv to update
%Impedance 
Zhyb = P./Q_tilde;

%Freq range to display
fmax = fstop; %fs/2;
ifmax=find(round(freq_v-fmax)>=0,1);
freqv_disp = freq_v(1:ifmax);

figure,
plotg(freqv_disp, P(1:ifmax)/20e-6) %/20e-6 to have dB SPL with p0 = threshold of human hearing.
hold on,
plotg(freqv_disp, UAO(1:ifmax),'--r') % dBV has ref 1V, so no scaling here
grid on,
title('Spectra of the measured pressure and the input voltage')
legend('Pressure (dB SPL)','Volts (dBV)')

figure,
plotgp(freqv_disp,2*Zhyb((1:ifmax))/Zc,'-b') 
subplot 211,
hold on,
stem(freqv_disp(abs(angle(Zhyb(1:ifmax)))<0.2),20*log10(2*abs(Zhyb(abs(angle(Zhyb(1:ifmax)))<0.2))/Zc),'-g')
stem(freqv_disp(abs(angle(Zhyb(1:ifmax))-pi)<0.2),20*log10(2*abs(Zhyb(abs(angle(Zhyb(1:ifmax))-pi)<0.2))/Zc),'--r')


%% Sound

fs_sound = fs;%44100;
p_sound=resample(pressure_Pa-mean(pressure_Pa),fs_sound,round(fs));% only exactly the right timing when round(fs)=fs ...
p_sound = p_sound/max(abs(p_sound));
% Play the sound, note: we here some aliasing -> due to the Matlab sound
% there is no aliasing in the p_sound vector !
sound(p_sound,fs_sound);
wavwrite(p_sound,fs_sound,'p');