

/*
 * Include Files
 *
 */
#if defined(MATLAB_MEX_FILE)
#include "tmwtypes.h"
#include "simstruc_types.h"
#else
#include "rtwtypes.h"
#endif

/* %%%-SFUNWIZ_wrapper_includes_Changes_BEGIN --- EDIT HERE TO _END */
#include <math.h>
/* %%%-SFUNWIZ_wrapper_includes_Changes_END --- EDIT HERE TO _BEGIN */
#define u_width 1
#define y_width 1
/*
 * Create external references here.  
 *
 */
/* %%%-SFUNWIZ_wrapper_externs_Changes_BEGIN --- EDIT HERE TO _END */
/* extern double func(double a); */
/*I use this for global variable declarations too!*/
real_T alpha_wrx;
real_T alpha_Qrx;
real_T p0;
real_T fs_over_wr;
real_T fs_over_wr_p2;
real_T a [3];
real_T b1;
real_T y;
/* %%%-SFUNWIZ_wrapper_externs_Changes_END --- EDIT HERE TO _BEGIN */

/*
 * Output functions
 *
 */
void reed_discrete_LP_filter_hitting_reed_Outputs_wrapper(const real_T *p,
			const real_T *wr,
			const real_T *Qr,
			real_T *y0,
			const real_T *xD,
			const real_T  *fs, const int_T  p_width0,
			const real_T  *alpha_wr, const int_T  p_width1,
			const real_T  *alpha_Qr, const int_T  p_width2,
			const real_T  *yth, const int_T  p_width3,
			const real_T  *yb, const int_T  p_width4,
			const real_T  *ae, const int_T  p_width5)
{
/* %%%-SFUNWIZ_wrapper_Outputs_Changes_BEGIN --- EDIT HERE TO _END */
/* This sample sets the output equal to the input
         y0[0] = u0[0];
For complex signals use: y0[0].re = u0[0].re;
                         y0[0].im = u0[0].im;
                         y1[0].re = u1[0].re;
                         y1[0].im = u1[0].im;*/

alpha_wrx = 1;
alpha_Qrx = 1;
p0 = p[0];

/* Look at the previous dispacement output to set the stiffness and friction coefficients.
 *When the reed starts beating, the stiffness and friction increases, but gradually (Vergez2000)*/
if (xD[0]<*yb) {
        alpha_wrx = *alpha_wr;
        alpha_Qrx = *alpha_Qr;
}
/* Linearly interpolate between the beating and non-beating states, starting from a threshold yth*/
else if (xD[0]<*yth) {
        alpha_wrx = (xD[0]- *yth)*(1- *alpha_wr)/(*yth-*yb) + 1;
        alpha_Qrx = (xD[0]- *yth)*(1- *alpha_Qr)/(*yth-*yb) + 1;
}

fs_over_wr = *fs/(wr[0]*alpha_wrx);
fs_over_wr_p2 = pow(fs_over_wr,2);
p0 = (p0- *yth)/ pow(alpha_wrx,2) + *yth;/*the pressure offset is to correct for the added stiffness + y-offset. -> see 19/8/2015 in my notebook.*/

/*This 2nd order filter is drawn from Guillemain2005.
 *It is normalised by the stiffness factor and the discretisation scheme allowed to remove the "1-sample delay"...*/
a[0] = fs_over_wr_p2 + fs_over_wr/(2*Qr[0]*alpha_Qrx);
a[1] = (2*fs_over_wr_p2 -1)/a[0];
a[2] = (fs_over_wr/(2*Qr[0]*alpha_Qrx) - fs_over_wr_p2)/a[0];
b1 = 1/a[0];

y = b1*p0 + a[1]*xD[0] + a[2]*xD[1];
y0[0] = y - *yb;
/*Experiments for a smoother lip-opening.
 *Smoothing only in yb-yth zone, but: doesn't remove the noise...
if (*yb<y<*yth) {
    y0[0] = pow(y0[0],2)/ (*yth- *yb);
}*/
/* A complete quadratic function. eg: Msallam2000(->Copley), Bromage2010 !: use power with exponent between 1 and 2 */
if (y0[0]>0) {
    y0[0] = pow(y0[0],*ae);
}
/* %%%-SFUNWIZ_wrapper_Outputs_Changes_END --- EDIT HERE TO _BEGIN */
}

/*
  * Updates function
  *
  */
void reed_discrete_LP_filter_hitting_reed_Update_wrapper(const real_T *p,
			const real_T *wr,
			const real_T *Qr,
			const real_T *y0,
			real_T *xD,
			const real_T  *fs,  const int_T  p_width0,
			const real_T  *alpha_wr,  const int_T  p_width1,
			const real_T  *alpha_Qr,  const int_T  p_width2,
			const real_T  *yth,  const int_T  p_width3,
			const real_T  *yb,  const int_T  p_width4,
			const real_T  *ae,  const int_T  p_width5)
{
  /* %%%-SFUNWIZ_wrapper_Update_Changes_BEGIN --- EDIT HERE TO _END */
xD[1] = xD[0];
xD[0] = y;
/* %%%-SFUNWIZ_wrapper_Update_Changes_END --- EDIT HERE TO _BEGIN */
}
